# GNU Enterprise Application Server - Class Repository: Base classes
#
# Copyright 2003 Free Software Foundation
#
# This file is part of GNU Enterprise.
#
# GNU Enterprise is free software; you can redistribute it 
# and/or modify it under the terms of the GNU General Public 
# License as published by the Free Software Foundation; either 
# version 2, or (at your option) any later version.
#
# GNU Enterprise is distributed in the hope that it will be 
# useful, but WITHOUT ANY WARRANTY; without even the implied 
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
# PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public 
# License along with program; see the file COPYING. If not, 
# write to the Free Software Foundation, Inc., 59 Temple Place 
# - Suite 330, Boston, MA 02111-1307, USA.
#
# $Id: Base.py,v 1.3 2003/10/21 12:26:50 jvetter Exp $

import types

# =============================================================================
# Class-wrapper for dictionaries
# =============================================================================
class BaseDictionary:

  # ---------------------------------------------------------------------------
  # Construct an instance 
  # ---------------------------------------------------------------------------
  def __init__ (self, session, defaultCache = None):
    self.session = session
    self._cache  = {}

    if defaultCache:
      self._cache = defaultCache

  # ---------------------------------------------------------------------------
  # Return all keys 
  # ---------------------------------------------------------------------------
  def keys (self):
    keys = self._cache.keys ()
    keys.sort ()
    return keys

  # ---------------------------------------------------------------------------
  # Return all values
  # ---------------------------------------------------------------------------
  def values (self):
    keys = self.keys ()
    return map (self._cache.get, keys)

  # ---------------------------------------------------------------------------
  # Return all key-value-pairs
  # ---------------------------------------------------------------------------
  def items (self):
    return self._cache.items ()

  # ---------------------------------------------------------------------------
  # Check for a specified key
  # ---------------------------------------------------------------------------
  def has_key (self, key):
    return self._cache.has_key (key)

  # ---------------------------------------------------------------------------
  # Return the number of items in the dictionary
  # ---------------------------------------------------------------------------
  def __len__ (self):
    return len (self._cache)

  # ---------------------------------------------------------------------------
  # Route class-access to the underlying dictionary
  # ---------------------------------------------------------------------------
  def __getitem__ (self, key):
    return self.get (key)


# =============================================================================
# This class maps all dictionary-keys to properties
# =============================================================================
class BaseObject:
  def __init__ (self, data):
    self._data = data

  # ---------------------------------------------------------------------------
  # Undefined attributes must exist in data-dictionary
  # ---------------------------------------------------------------------------
  def __getattr__ (self, attr):
    try:
      value = self._data [attr]
      if isinstance (value, types.UnicodeType):
        return value.encode ('utf-8')
      else:
        return value

    except KeyError:
      raise AttributeError, attr

    except:
      raise

