# GNU Enterprise Application Server - Class Repository: Module
#
# Copyright 2003 Free Software Foundation
#
# This file is part of GNU Enterprise.
#
# GNU Enterprise is free software; you can redistribute it 
# and/or modify it under the terms of the GNU General Public 
# License as published by the Free Software Foundation; either 
# version 2, or (at your option) any later version.
#
# GNU Enterprise is distributed in the hope that it will be 
# useful, but WITHOUT ANY WARRANTY; without even the implied 
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
# PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public 
# License along with program; see the file COPYING. If not, 
# write to the Free Software Foundation, Inc., 59 Temple Place 
# - Suite 330, Boston, MA 02111-1307, USA.
#
# $Id: Module.py,v 1.9 2003/10/21 12:26:50 jvetter Exp $

import types
from Base import *
from Namespace import *
from Class import ClassDict

# =============================================================================
# A List Of Business Object Modules
# =============================================================================
class ModuleDict (BaseDictionary):

  # ---------------------------------------------------------------------------
  # This class behaves like a dictionary of all available modules with the
  # keys being the module names and the values being Module objects.
  # ---------------------------------------------------------------------------
  def __init__ (self, session, definition = None):
    BaseDictionary.__init__ (self, session)
    self.definition = definition
    self.classdict  = None

    if definition:
      self._updateCache ([module [1] for module in definition.modules ()])
    else:
      self._updateCache (self._requestData ())

  # ---------------------------------------------------------------------------
  # Integrate a list of business objects into the cache
  # ---------------------------------------------------------------------------
  def _updateCache (self, list):
    for item in list:
      module = self._module_from_cache (item)
      self._cache [module.fullName] = module

  # ---------------------------------------------------------------------------
  # Internal: Request modules from datasource (optional matching a name)
  # ---------------------------------------------------------------------------
  def _requestData (self, name = None):
    cond = []
    if name:
      cond = [["eq", ""], ["field", "gnue_name"], ["const", name]]
    return self.session.find ("gnue_module", cond, ["gnue_name"],
                             ["gnue_name", "gnue_id"])

  # ---------------------------------------------------------------------------
  # Internal: create an instance from a cached object
  # ---------------------------------------------------------------------------
  def _module_from_cache (self, item):
    return Module (self, item)

  # ---------------------------------------------------------------------------
  # Return a Module object if the given module exists
  # ---------------------------------------------------------------------------
  def get (self, name):
    # is the requested module cached? 
    if self.has_key (name):
      return self._cache [name]

    # since it isn't cached we've to ask for it
    list = self._requestData (name)

    # if we have an exact match extend the cache
    if len (list) == 1:
      self._updateCache (list)
      return self._cache [name]
      
    raise EModuleNotFound (name)



# =============================================================================
# A Business Object Module
# =============================================================================
class Module:

  def __init__ (self, modDict, object):
    # _object is the underlying business object
    self.modules  = modDict
    self._object  = object

    self.fullName = self.gnue_name

  # ---------------------------------------------------------------------------
  # Return an attribute's value
  # ---------------------------------------------------------------------------
  def __getattr__ (self, attr):
    if attr == 'classes':
      self.__dict__ ['classes'] = ClassDict (self.modules.session,
                                             self.modules, self)
      return self.classes

    else:
      value = getattr (self._object, attr)
      if isinstance (value, types.UnicodeType):
        return value.encode ('utf-8')
      else:
        return value
