# GNU Enterprise Application Server - List Object
#
# Copyright 2001-2003 Free Software Foundation
#
# This file is part of GNU Enterprise.
#
# GNU Enterprise is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either
# version 2, or (at your option) any later version.
#
# GNU Enterprise is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
# PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with program; see the file COPYING. If not,
# write to the Free Software Foundation, Inc., 59 Temple Place
# - Suite 330, Boston, MA 02111-1307, USA.
#
# $Id: geasList.py,v 1.31 2003/10/07 16:03:53 siesel Exp $

from gnue.common.datasources import GDataSource, GConditions
import geasInstance
import string

# =============================================================================
# List class
# =============================================================================

class geasList:

  # ---------------------------------------------------------------------------
  # Initalize
  # ---------------------------------------------------------------------------

  def __init__ (self, session, classdef, propertylist, sortorder):
    self._session = session
    self._classdef = classdef
    self._prefetch = propertylist       # property names to be prefetched
    self._sort = sortorder              # property names of the sort key

    self._prefColumns = []              # (db) column names to be prefetched
    self._sortColumns = []              # (db) column names of the sort key

    # test if all properties in prefetch list are valid and find column names
    for propertyname in self._prefetch:
      propertydef = self._classdef.properties [propertyname]
      self._prefColumns.append (propertydef.column)

    # test if all properties in sort list are valid and find column names
    for propertyname in self._sort:
      propertydef = self._classdef.properties [propertyname]
      self._sortColumns.append (propertydef.column)

    # prepare attributes of the datasource
    attributes = {}
    attributes ["name"]     = ""
    attributes ["database"] = self._session.database
    attributes ["table"]    = self._classdef.table

    if self._sortColumns != []:
      attributes ["order_by"] = string.joinfields (self._sortColumns, ",")

    # create the datasource
    self._datasource = GDataSource.DataSourceWrapper (
      connections = self._session.connections,
      attributes = attributes,
      fields = self._prefColumns)

    # enable unicode mode for the datasource
    self._datasource._dataObject._unicodeMode=1

  # ---------------------------------------------------------------------------
  # Populate the list with data from the database backend
  # ---------------------------------------------------------------------------
  # conditions = a list consisting of condition elements written down in
  # prefix notation i.e. (a=12) AND (c=121) -> AND = a 12 = c 121
  # every element of the list is a tuple containing the following parts:
  # type (eq,and,const...)  value  i.e. ('eq','') or ('const',12)
  # ---------------------------------------------------------------------------

  def populate (self, conditions):
    # TODO: translate property names to column names in conditions
    # now building GCondition tree    
    conditionTree = GConditions.buildTreeFromPrefix (conditions)
    self._resultset = self._datasource.createResultSet (conditionTree)

  # ---------------------------------------------------------------------------
  # Populate the list with a single instance with given gnue_id, and return
  # that instance, return an exception if not found
  # ---------------------------------------------------------------------------

  def find (self, object_id):
    self.populate ([['eq', ''], ['field', 'gnue_id'], ['const', object_id]])
    result = self.firstInstance ()
    if result == None:
      raise Exception, "Can't find object ID '%s' in class '%s'" % \
                       (object_id, self._classdef.fullName)
    return result

  # ---------------------------------------------------------------------------
  # Populate the list with a single empty instance, and return that instance
  # ---------------------------------------------------------------------------

  def newInstance (self):
    self._resultset = self._datasource.createEmptyResultSet ()
    self._resultset.insertRecord ()
    return geasInstance.geasInstance (self, self._resultset.current,
                                      self._classdef)

  # ---------------------------------------------------------------------------
  # Get the first instance in the list
  # ---------------------------------------------------------------------------

  def firstInstance (self):
    if self._resultset.firstRecord () != None:
      return geasInstance.geasInstance (self,  self._resultset.current,
                                        self._classdef)
    else:
      return None

  # ---------------------------------------------------------------------------
  # Get the next instance in the list
  # ---------------------------------------------------------------------------

  def nextInstance (self):
    if self._resultset.nextRecord () != None:
      return geasInstance.geasInstance (self, self._resultset.current,
                                        self._classdef)
    else:
      return None

  # ---------------------------------------------------------------------------
  # Get the length of the list (the number of entries)
  # ---------------------------------------------------------------------------

  def count (self):
    return self._resultset.getRecordCount ()

  # ---------------------------------------------------------------------------
  # Fetch data from the database backend
  # ---------------------------------------------------------------------------

  def fetch (self, start, count):
    # negative value for start means starting from the end of the list
    if start < 0:
      start = self.count () + start
    result = []
    c = 0
    instance = self.firstInstance ()
    while (instance != None) and (c < start + count):
      if c >= start:
        result.append (instance.get (self._prefetch))
      c += 1
      instance = self.nextInstance ()
    return result
