# GNU Enterprise Application Server - Session Manager Object
#
# This file is part of GNU Enterprise.
#
# GNU Enterprise is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either
# version 2, or (at your option) any later version.
#
# GNU Enterprise is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
# PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with program; see the file COPYING. If not,
# write to the Free Software Foundation, Inc., 59 Temple Place
# - Suite 330, Boston, MA 02111-1307, USA.
#
# Copyright 2001-2003 Free Software Foundation
#
# $Id: geasSessionManager.py,v 1.17 2003/10/21 12:26:49 jvetter Exp $

import geasSession
import geasAuthentication
import classrep

# =============================================================================
# Exceptions
# =============================================================================

class AuthError (Exception):
  pass

# =============================================================================
# Session Manager class
# =============================================================================

class geasSessionManager:

  # ---------------------------------------------------------------------------
  # Initalize
  # ---------------------------------------------------------------------------

  def __init__ (self, connections):
    self._connections = connections
    self._sessNo = 0
    self._sessions = {}
    self._buildInternalSession ()
    classrep.init (self)
    self.modules = classrep.modules
    self.classes = classrep.classes

    # TODO: load default authagent depending on config setting
    self._authAdapter = geasAuthentication.geasDBAuthAgent (self._internalSession)

  # ---------------------------------------------------------------------------
  # Build an internal session
  # ---------------------------------------------------------------------------

  def _buildInternalSession (self):
    self._internalSession = geasSession.geasSession (self._connections,
                                                     geasAuthentication.geasAuthAgent())
    self._internalSession.login (None,None) # fake login
    self._sessions [self._internalSession] = self._internalSession

  # ---------------------------------------------------------------------------
  # Find a session from session ID
  # ---------------------------------------------------------------------------

  def _getSession (self, sess_id):
    if self._sessions.has_key (sess_id):
      return self._sessions [sess_id]
    else:
      raise Exception, "Can't find a session with ID '%s'" % sess_id

  # ---------------------------------------------------------------------------
  # official API functions
  # ---------------------------------------------------------------------------

  def open (self, authentication):
    sess = geasSession.geasSession (self._connections, self._authAdapter)

    if not sess.login (authentication ['user'],
                       authentication ['password']):
      raise AuthError
    # TODO use a better session ID than this one
    self._sessNo = self._sessNo + 1
    self._sessions [self._sessNo] = sess
    return self._sessNo

  def close (self, session_id, commit):
    s = self._getSession (session_id)
    if commit:
      s.commit ()
    s.logout ()
    del self._sessions [session_id]

  def commit (self, session_id):
    s = self._getSession (session_id)
    s.commit ()

  def rollback (self, session_id):
    s = self._getSession (session_id)
    s.rollback ()

  def request (self, session_id, classname, conditions, sortorder,
               propertylist):
    s = self._getSession (session_id)
    return s.request (classname, conditions, sortorder, propertylist)

  def count (self, session_id, list_id):
    s = self._getSession (session_id)
    return s.count (list_id);

  def fetch (self, session_id, list_id, start, count, close=0):
    s = self._getSession (session_id)
    return s.fetch (list_id, start, count)

  def load (self, session_id, classname, obj_id_list, propertylist):
    s = self._getSession (session_id)
    return s.load (classname, obj_id_list, propertylist)

  def store (self, session_id, classname, obj_id_list, propertylist, data):
    s = self._getSession (session_id)
    return s.store (classname, obj_id_list, propertylist, data)

  def call (self, session_id, classname, obj_id_list, methodname, parameters):
    s = self._getSession (session_id)
    return s.call (classname, obj_id_list, methodname, parameters)

  def delete (self, session_id, classname, obj_id_list):
    s = self._getSession (session_id)
    return s.delete (classname, obj_id_list)
