# GNU Enterprise Application Server - Trigger Settings
#
# Copyright 2001-2003 Free Software Foundation
#
# This file is part of GNU Enterprise.
#
# GNU Enterprise is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either
# version 2, or (at your option) any later version.
#
# GNU Enterprise is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
# PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with program; see the file COPYING. If not,
# write to the Free Software Foundation, Inc., 59 Temple Place
# - Suite 330, Boston, MA 02111-1307, USA.
#
# $Id: geasTrigger.py,v 1.6 2003/09/01 18:01:45 reinhard Exp $

import string,new

# =============================================================================
# Basic Trigger Class ( abstract )
# =============================================================================

class geasTriggerManager:

  # ---------------------------------------------------------------------------
  # Initalize
  # ---------------------------------------------------------------------------

  def __init__ (self):
    pass

  # ---------------------------------------------------------------------------
  # get named trigger X
  # the returned trigger is an executable object, which can be called directly
  # ---------------------------------------------------------------------------

  def getTriggerByName(self, triggername):
    return None

  # ---------------------------------------------------------------------------
  # get named trigger X
  # the returned trigger is an executable object, which can be called directly
  # ---------------------------------------------------------------------------

  def getTriggerByEvent(self, triggerevent):
    return None


# =============================================================================
# Database Authentification Agent
# =============================================================================

class geasPythonDBTriggerMg(geasTriggerManager):

  # ---------------------------------------------------------------------------
  # Initalize
  # ---------------------------------------------------------------------------

  def __init__ (self,session):
    # creation and populating the triggerlist list should be
    # moved here
    self._session=session

  # ---------------------------------------------------------------------------
  # get named trigger X
  # this trigger is an executable object, which can be called directly
  # ---------------------------------------------------------------------------

  def getTriggerByName(self, triggername):
    triggerList = self._session.createList ("appserver_pytrigger")

    triggerList.setPrefetch (["name", "event", "code"])
    triggerList.setConditions ([['eq', ''],
                             ['field', 'name'],
                             ['const', triggername ]])
    triggerList.populate ()

    instance = triggerList.firstInstance ()

    # no such trigger
    if instance==None:
      # log something
      print _("Trigger '%s' does not exist.") % triggername

      return None

    code=instance.get('code')
    # format code

    ccode=compile (code,'<string>','exec')
    myfunc=new.function(ccode, {'session':self._session}, triggername)

    return myfunc

  # ---------------------------------------------------------------------------
  # get named trigger X
  # this trigger is an executable object, which can be called directly
  # ---------------------------------------------------------------------------

  def getTriggerByEvent(self, eventname):
    triggerList = self._session.createList ("appserver_pytrigger")

    triggerList.setPrefetch (["name", "event", "code"])
    triggerList.setConditions ([['eq', ''],
                             ['field', 'event'],
                             ['const', eventname ]])
    triggerList.populate ()

    instance = triggerList.firstInstance ()

    # no such trigger
    if instance==None:
      # log something
#     print _("Trigger for event '%s' does not exist.") % eventname

      return None

    code=instance.get('code')
    # format code

    ccode=compile (code,'<string>','exec')
    myfunc=new.function(ccode, {'session':self._session}, instance.get('name'))

    return myfunc

