# GNU Enterprise Application Server - Language interface: list of objects
#
# Copyright 2003 Free Software Foundation
#
# This file is part of GNU Enterprise.
#
# GNU Enterprise is free software; you can redistribute it 
# and/or modify it under the terms of the GNU General Public 
# License as published by the Free Software Foundation; either 
# version 2, or (at your option) any later version.
#
# GNU Enterprise is distributed in the hope that it will be 
# useful, but WITHOUT ANY WARRANTY; without even the implied 
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
# PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public 
# License along with program; see the file COPYING. If not, 
# write to the Free Software Foundation, Inc., 59 Temple Place 
# - Suite 330, Boston, MA 02111-1307, USA.
#
# $Id: ObjectList.py,v 1.7 2003/10/19 17:55:56 reinhard Exp $

import types
from Object import newObject, rpc_to_native

CACHE_STEP = 10

def newList (server, sessionId, classname, cond, sort, properties):
  return ObjectList (server, sessionId, classname, cond, sort, properties)

# ===========================================================================
# Class: ObjectList
# ===========================================================================
class ObjectList:

  # -------------------------------------------------------------------------
  # Constructor
  # -------------------------------------------------------------------------
  def __init__ (self, server, sessionId, classname, cond, sort, properties):
    self._server     = server
    self._session_id = sessionId
    self._cache      = []
    self.classname   = classname
    self.conditions  = cond
    self.sortOrder   = sort
    self.properties  = properties

    # prepare the sequence-cache
    self._buildCache ()


  # -------------------------------------------------------------------------
  # Return the number of elements in this sequence
  # -------------------------------------------------------------------------
  def __len__ (self):
    return self._length

  # -------------------------------------------------------------------------
  # Return a slice or a specific element of this sequence
  # -------------------------------------------------------------------------
  def __getitem__ (self, index):
    try:
      if type (index) == types.SliceType:
        return self._cache [index.start:index.stop]
      else:
        return self._cache [index]
    except:
      # try to populate the cache for missing entries
      if self._populateCache():

        # if successful try to return entry/entries again
        return self.__getitem__(index)
      
      else:
        raise IndexError
      
      
      

  # -------------------------------------------------------------------------
  # Create a cache for this sequence
  # -------------------------------------------------------------------------
  def _buildCache (self):
    self.list_id = self._server.request (self._session_id, self.classname,
                                         self.conditions, self.sortOrder, 
					 self.properties)
    # fix for gnue-common 0.5.1 release:
    #  - don't depend on RecordSet.Count functionality
    # self._length = self._server.count (self._session_id, self.list_id)
    self._cache  = []

    self._populateCache()

      
  # -------------------------------------------------------------------------
  # Populate Cache
  # -------------------------------------------------------------------------
  def _populateCache (self):
    rset = self._server.fetch (self._session_id, self.list_id, 
                               len (self._cache), CACHE_STEP, 0)    
    types = self._server.load (self._session_id, self.classname, [''],
                               self.properties)
    for row in rset:
      ## Convert array into dictonary
      i = 0
      objData = {}

      for p in self.properties:
        objData [p] = rpc_to_native (row [i+1], types [0] [i])
        i += 1
        
      objData ["gnue_id"] = row [0]
      obj = newObject (self._server, self._session_id, self.classname, objData)
      self._cache.append (obj)

    # fix for gnue-common 0.5.1 release (will be removed for later releases 
    # of gnue-common)
    self._length = len (self._cache)

    # return number of updated rows
    return len (rset)
    

  # -------------------------------------------------------------------------
  # Extend the cached sequence
  # -------------------------------------------------------------------------
  def append (self, value):
    self._cache.append (value)

