# GNU Enterprise Application Server - Language interface session
#
# Copyright 2003 Free Software Foundation
#
# This file is part of GNU Enterprise.
#
# GNU Enterprise is free software; you can redistribute it 
# and/or modify it under the terms of the GNU General Public 
# License as published by the Free Software Foundation; either 
# version 2, or (at your option) any later version.
#
# GNU Enterprise is distributed in the hope that it will be 
# useful, but WITHOUT ANY WARRANTY; without even the implied 
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
# PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public 
# License along with program; see the file COPYING. If not, 
# write to the Free Software Foundation, Inc., 59 Temple Place 
# - Suite 330, Boston, MA 02111-1307, USA.
#
# $Id: Session.py,v 1.9 2003/08/16 17:07:16 siesel Exp $

from ObjectList import newList
from Object import newObject

# ===========================================================================
# CLASS session: implement a session of the language interface
# ===========================================================================

class Session:

  # -------------------------------------------------------------------------
  # Constructor:
  # -------------------------------------------------------------------------

  def __init__ (self, sessionManager, user, password):
    self._sm = sessionManager
    self._session_id = self._sm.open ({'user': user, 'password': password})

  # -------------------------------------------------------------------------
  # Close the session 
  # -------------------------------------------------------------------------
  def close (self):
    if self._sm:
      self._sm.close (self._session_id, None)
      self._sm = self._session_id = None

  # -------------------------------------------------------------------------
  # Commit the current transaction
  # -------------------------------------------------------------------------
  def commit (self):
    if self._session_id:
      self._sm.commit (self._session_id)

  # -------------------------------------------------------------------------
  # Revoke the current transaction
  # -------------------------------------------------------------------------
  def rollback (self):
    if self._session_id:
      self._sm.rollback (self._session_id)

  # -------------------------------------------------------------------------
  # Set the default context
  # -------------------------------------------------------------------------
  def setcontext (self, context):
    if self._session_id:
      self._sm.setcontext (self._session_id, context)

  # -------------------------------------------------------------------------
  # Return a collection of 'classname' matching the given arguments
  # -------------------------------------------------------------------------
  def find (self, classname, cond=[], sort=['gnue_id'], properties=[]):
    return newList (self._sm, self._session_id, classname, cond, sort,
                    properties)

  # -------------------------------------------------------------------------
  # Create a new instance of classname
  # -------------------------------------------------------------------------
  def new (self, classname):
    return newObject (self._sm, self._session_id, classname)


class InternalSession(Session):

  # -------------------------------------------------------------------------
  # Constructor:
  # -------------------------------------------------------------------------

  def __init__ (self, sessionManager):
    self._sm = sessionManager
    self._session_id = self._sm._internalSession
